package org.mathcs.ibookmark;

import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.app.Activity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.DialogInterface.OnCancelListener;
import android.content.Intent;
import android.view.ActionMode;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.widget.AbsListView;
import android.widget.AdapterView;
import android.widget.ListView;
import android.widget.Toast;

public class BookmarkActivity extends Activity
{
	public final static String VERSION = "1.0";
	public final static String KEY_URL = "url";
	public final static String KEY_TITLE = "title";
	public final static String KEY_INDEX = "index";
	public final static String DEFAULT_URL = "www.shu.edu";
	public final static String BOOKMARK_FILE_NAME = "bookmarks.txt";
	public final static String BOOKMARK_FILE_URL = "http://pirate.shu.edu/~wachsmut/Temp/bookmarks.txt";
	public final static int ITEM_ADD_REQUEST_CODE = 0;
	public final static int ITEM_EDIT_REQUEST_CODE = 1;

	private BookmarkAdapter data;
	private BookmarkLoader loader;
	private ListView list;
	
	@Override
	protected void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_bookmark);

		// Setting default preferences from XML file. If last parameter is 'false', the
		// default values are set exactly once.
		PreferenceManager.setDefaultValues(this, R.xml.preferences, false);
		
		data = new BookmarkAdapter(this);
		
		list = (ListView) this.findViewById(R.id.id_list);
		list.setAdapter(data);

		list.setChoiceMode(ListView.CHOICE_MODE_MULTIPLE_MODAL);
		list.setDrawSelectorOnTop(true);
        
		list.setOnItemClickListener(new AdapterView.OnItemClickListener()
			{
				@Override
				public void onItemClick(AdapterView<?> adapter, View view, int pos,
						long arg3)
				{
					startBookmark(pos);
				}
			});
		list.setMultiChoiceModeListener(new AbsListView.MultiChoiceModeListener()
		{
			@Override
			public boolean onCreateActionMode(ActionMode mode, Menu menu)
			{
				// Inflate the menu for the CAB
				MenuInflater inflater = mode.getMenuInflater();
				inflater.inflate(R.menu.context, menu);
				return true;
			}

			@Override
			public void onItemCheckedStateChanged(ActionMode mode, int position, long id, boolean checked)
			{
				// Here you can do something when items are selected/de-selected,
				// such as update the title in the CAB
				data.get(position).setSelected(checked);
				data.notifyDataSetChanged();
				mode.setTitle(list.getCheckedItemCount() + " selected");
			}

			@Override
			public boolean onActionItemClicked(ActionMode mode, MenuItem item)
			{
				// Respond to clicks on the actions in the CAB
				if (item.getItemId() == R.id.id_menu_del)
					deleteSelectedItems();
				else if (item.getItemId() == R.id.id_menu_edit)
					editSelectedItem();
				mode.finish();
				return false;
			}

			@Override
			public void onDestroyActionMode(ActionMode mode)
			{
				// Here you can make any necessary updates to the activity when CAB is removed. 
				data.deselectAll();
				data.notifyDataSetChanged();
			}

			@Override
			public boolean onPrepareActionMode(ActionMode mode, Menu menu)
			{
				// Here you can perform updates to the CAB due to an invalidate() request
				return false;
			}
		});
		
		loadData();
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu)
	{
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.bookmark, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item)
	{
		if (item.getItemId() == R.id.id_menu_add)
			addItem();
		else if (item.getItemId() == R.id.action_settings)
			showSettings();
		return super.onOptionsItemSelected(item);
	}

	@Override
	public void onPause()
	{
		super.onPause();
		if ((data != null) && (data.isDirty()))
		{
			saveData();
		}
	}

	@Override
	public void onResume()
	{
		super.onResume();
//		loadData();
	}

	@Override
	public void onActivityResult(int requestCode, int resultCode, Intent intent) 
	{
		super.onActivityResult(requestCode,  resultCode, intent);
		// requestCode is ignored since we only use one. But resultCode is used!
		if (resultCode == RESULT_CANCELED)
			;
		else if (resultCode == RESULT_OK)
		{
			String title = intent.getStringExtra(KEY_TITLE);
			String url = intent.getStringExtra(KEY_URL);
			int index = intent.getIntExtra(KEY_INDEX, -1);
			if (index < 0)
				data.add(new Bookmark(title, url));
			else
				data.update(new Bookmark(title, url), index);
			saveData();
		}
	}
	
	private void startBookmark(int position)
	{
		Intent intent = new Intent(this, UrlViewer.class);
		intent.putExtra(KEY_URL, data.get(position).getUrl());
		startActivity(intent);
	}

	private void addItem()
	{
		Intent intent = new Intent(this, BookmarkItemEdit.class);
		startActivityForResult(intent, ITEM_ADD_REQUEST_CODE);
	}
	
	private int findFirstSelectedIndex()
	{
		int i = 0;
		while ((i < data.getCount()) && (!data.get(i).isSelected()))
			i++;
		if (i < data.getCount())
			return i;
		else
			return -1;
	}
	
	private void editSelectedItem()
	{
		int index = findFirstSelectedIndex();
		if (index >= 0)
		{
			Bookmark item = data.get(index);
			Intent intent = new Intent(this, BookmarkItemEdit.class);
			intent.putExtra(KEY_URL, item.getUrl());
			intent.putExtra(KEY_TITLE, item.getTitle());
			intent.putExtra(KEY_INDEX, index);
			startActivityForResult(intent, ITEM_EDIT_REQUEST_CODE);
		}
	}
	
	private void deleteSelectedItems()
	{
		for (int i = data.getCount()-1; i >= 0; i--)
		{
			if (data.get(i).isSelected())
				data.remove(i);
		}
		data.notifyDataSetChanged();
	}

	private void showSettings()
	{
		Intent intent = new Intent(this, SettingsActivity.class);
		startActivityForResult(intent, 0);
	}
	
	public void resetPrefsToDefaultValues()
	{
	}
	
	private void saveData()
	{
		try
		{
			PrintWriter out = new PrintWriter(new OutputStreamWriter(
					openFileOutput(BOOKMARK_FILE_NAME, Context.MODE_PRIVATE)));
			data.saveBookmarks(out);
			out.close();
		} 
		catch (Exception ex)
		{
			Toast.makeText(this, "Error saving data: " + ex.getMessage(), Toast.LENGTH_SHORT).show();
		}
	}

	private void cancelLoading()
	{
    	if (loader != null)
    		loader.cancelLoading();
	}
	
	private void loadData()
	{
		ProgressDialog progress = new ProgressDialog(this);
		progress.setMessage("Loading bookmarks, please wait ...");
		progress.setIndeterminate(true);
		progress.setOnCancelListener(new OnCancelListener()
				{

					@Override
					public void onCancel(DialogInterface dialog)
					{
						cancelLoading();
					}
					
				});
		cancelLoading();

		loader = new BookmarkLoader(data, progress);
		loader.execute(BOOKMARK_FILE_URL);
	
	}
	/*
	private void loadData()
	{
		try
		{
			URL url = new URL(BOOKMARK_FILE_URL);
			BufferedReader in = new BufferedReader(new InputStreamReader(url.openStream()));
			data.loadBookmarks(in);
			data.notifyDataSetChanged();

			in.close();
		} 
		catch (Exception ex)
		{
			Toast.makeText(this, "Error loading data: " + ex,
					Toast.LENGTH_SHORT).show();
		}
	}
	*/
}
